/* 
 * This file is part of the ARM Cortex-M Fault distribution.
 * Copyright (c) 2018 Martin Wende.
 * 
 * This program is free software: you can redistribute it and/or modify  
 * it under the terms of the GNU General Public License as published by  
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef MPU_H
#define MPU_H

/** 
	enum fr Regionsgre
*/
enum mpu_regionsize {
	MPU_REGS_32BYTE = 	4,
	MPU_REGS_64BYTE = 	5,
	MPU_REGS_128BYTE = 	6,
	MPU_REGS_256BYTE = 	7,
	MPU_REGS_512BYTE = 	8,
	MPU_REGS_1KBYTE = 	9,
	MPU_REGS_2KBYTE = 	10,
	MPU_REGS_4KBYTE = 	11,
	MPU_REGS_8KBYTE = 	12,
	MPU_REGS_16KBYTE = 	13,
	MPU_REGS_32KBYTE = 	14,
	MPU_REGS_64KBYTE = 	15,
	MPU_REGS_128KBYTE = 16,
	MPU_REGS_256KBYTE = 17,
	MPU_REGS_512KBYTE = 18,
	MPU_REGS_1MBYTE = 	19,
	MPU_REGS_2MBYTE = 	20,
	MPU_REGS_4MBYTE = 	21,
	MPU_REGS_8MBYTE = 	22,
	MPU_REGS_16MBYTE = 	23,
	MPU_REGS_32MBYTE = 	24,
	MPU_REGS_64MBYTE = 	25,
	MPU_REGS_128MBYTE = 26,
	MPU_REGS_256MBYTE = 27,
	MPU_REGS_512MBYTE = 28,
	MPU_REGS_1GBYTE = 	29,
	MPU_REGS_2GBYTE = 	30,
	MPU_REGS_4GBYTE = 	31,
};

/** 
	enum fr Regionsattribute
*/
enum mpu_regionattributes {
				//TEX_S_C_B
	MPU_FLASH = 0b000010,	/**< fr Flash */
	MPU_RAMI  = 0b000110,	/**< fr internen RAM */
	MPU_RAME  = 0b000111,	/**< fr externen RAM */
	MPU_PERI  = 0b000101,	/**< fr Peripherie */
};

/** 
	enum fr Zugriffsrechte
*/
enum mpu_permissions {
	MPU_NO_ACC  = 0b000,	/**< kein Zugriff */
	MPU_KRW_UNO = 0b001,	/**< Kernel RW, User kein Zugriff */
	MPU_KRW_URO = 0b010,	/**< Kernel RW, User nur lesen */
	MPU_KRW_URW = 0b011,	/**< Kernel RW, User RW */
	MPU_KRO_UNO = 0b101,	/**< Kernel nur lesen, User kein Zugriff */
	//MPU_KRO_URO = 0b110,
	MPU_KRO_URO = 0b111,	/**< Kernel nur lesen, User nur lesen */
};

/** \brief MPU Ein- oder Ausschalten

	\param onoff 1 -> on; 0 -> off
*/
void mpu_onoff(unsigned int onoff);

/**	\brief Region eines MPU Schutzbereiches einstellen

	\param region Nummer der Region [0 ... 7]
	\param baseaddr Startadresse der Region, muss ein vielfaches von mpu_regionsize sein
	\param mpu_regionsize Gre der MPU Region
	\param subregionbits Bitmaske von 8 Bits, erst nutzbar ab mehr als 128 Byte Regionsgre
	\param regionattributes Attribute der Region wie zB Cachebarkeit \see mpu_regionattributes
	\param permissions ZUgriffsrechte auf diese MPU Region \see mpu_permissions
	\param executable 1 -> Code ausfhrbar; 0 -> Abort bei Instructionfetch
*/
void mpu_set_region(	
		unsigned int region, 
		unsigned int baseaddr, 					//startet bei vielfachen von size
		enum mpu_regionsize regionsize_bits,	//regionsize_bits -> bytes in 2^(size+1)
		unsigned int subregionbits,				//Bitmaske von 8 Bits, erst nutzbar ab mehr als 128 Byte Regionsgre
		enum mpu_regionattributes regionattributes, 
		enum mpu_permissions permissions,
		unsigned int executable);

#endif//MPU_H