/* 
 * This file is part of the ARM Cortex-M Fault distribution.
 * Copyright (c) 2018 Martin Wende.
 * 
 * This program is free software: you can redistribute it and/or modify  
 * it under the terms of the GNU General Public License as published by  
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include "nvic.h"

#include <stdint.h>

// Basisadressen der NVIQ IRQ Nummer Regionen
static volatile uint32_t * const NVIC_SET_ENABLE   = (uint32_t *)0xE000E100;
static volatile uint32_t * const NVIC_CLEAR_ENABLE = (uint32_t *)0xE000E180;
static volatile uint32_t * const NVIC_CLEAR_PEND   = (uint32_t *)0xE000E280;
static volatile uint8_t * const NVIC_PRIORITY = (uint8_t *)0xE000E400;

#define IRQ_NBRS_PER_REG 32

static void nvic_setbit(volatile uint32_t * const base, uint32_t irq_nbr);

void nvic_enable_irq(uint32_t irq_nbr, uint8_t preempt_prio, uint8_t sub_prio){
	
	// Zuerst Prio zusammensetzen (default: 4bit prio, 4bit preempt)
	uint8_t prio = preempt_prio << NVICIRQ_PREEMPTBITS;
	prio |= sub_prio & NVICIRQ_SUBPRIOMASK;
	NVIC_PRIORITY[irq_nbr] = prio;
	
	// IRQ danach einschalten
	nvic_setbit(NVIC_SET_ENABLE, irq_nbr);
}

void nvic_disable_irq(uint32_t irq_nbr){

	nvic_setbit(NVIC_CLEAR_ENABLE, irq_nbr);
}

void nvic_pending_clear(uint32_t irq_nbr){

	nvic_setbit(NVIC_CLEAR_PEND, irq_nbr);
}

static void nvic_setbit(volatile uint32_t * const base, uint32_t irq_nbr){
	
	// passende Registeradresse finden
	uint32_t arr_index = irq_nbr/IRQ_NBRS_PER_REG;
	
	// Bit in Register finden
	uint32_t shift = irq_nbr - (arr_index*IRQ_NBRS_PER_REG);
	
	// Zugriff
	base[arr_index] = 1 << shift;
}