/* 
 * This file is part of the ARM Cortex-M Fault distribution.
 * Copyright (c) 2018 Martin Wende.
 * 
 * This program is free software: you can redistribute it and/or modify  
 * it under the terms of the GNU General Public License as published by  
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef NVIC_H
#define NVIC_H

#include <stdint.h>

#define NVICIRQ_SUBPRIOBITS	4
#define NVICIRQ_SUBPRIOMASK ((1<<NVICIRQ_SUBPRIOBITS) - 1)
#define NVICIRQ_PREEMPTBITS (8 - NVICIRQ_SUBPRIOBITS)

// fr F205, F405, F407, F411, F429, etc. (manches fehlt)
#define IRQ_NBR_DMA1_S0		11
#define IRQ_NBR_DMA1_S1		12
#define IRQ_NBR_DMA1_S2		13
#define IRQ_NBR_DMA1_S3		14
#define IRQ_NBR_DMA1_S4		15
#define IRQ_NBR_DMA1_S5		16
#define IRQ_NBR_DMA1_S6		17
#define IRQ_NBR_ADC			18
#define IRQ_NBR_TIM2		28
#define IRQ_NBR_TIM3		29
#define IRQ_NBR_TIM4		30
#define IRQ_NBR_SPI1		35
#define IRQ_NBR_SPI2		36
#define IRQ_NBR_USART1		37
#define IRQ_NBR_USART2		38
#define IRQ_NBR_USART3		39
#define IRQ_NBR_DMA1_S7		47
#define IRQ_NBR_TIM5		50
#define IRQ_NBR_SPI3		51
#define IRQ_NBR_UART4		52
#define IRQ_NBR_UART5		53
#define IRQ_NBR_DMA2_S0		56
#define IRQ_NBR_DMA2_S1		57
#define IRQ_NBR_DMA2_S2		58
#define IRQ_NBR_DMA2_S3		59
#define IRQ_NBR_DMA2_S4		60
#define IRQ_NBR_DMA2_S5		68
#define IRQ_NBR_DMA2_S6		69
#define IRQ_NBR_DMA2_S7		70
#define IRQ_NBR_USART6		71

/** \brief IRQ aktivieren mit 4Bit prio und 4bit preempt
	\param irq_nbr IRQ Nummer der Peripherie
	\param preempt_prio preemption prio
	\param sub_prio sub prio
*/
void nvic_enable_irq(uint32_t irq_nbr, uint8_t preempt_prio, uint8_t sub_prio);

/** \brief IRQ deaktivieren mit 4Bit prio und 4bit preempt
	\param irq_nbr IRQ Nummer der Peripherie
*/
void nvic_disable_irq(uint32_t irq_nbr);

/** \brief IRQ pending lschen
	\param irq_nbr IRQ Nummer der Peripherie
*/
void nvic_pending_clear(uint32_t irq_nbr);

#endif//NVIC_H