/* 
 * This file is part of the ARM Cortex-M Fault distribution.
 * Copyright (c) 2018 Martin Wende.
 * 
 * This program is free software: you can redistribute it and/or modify  
 * it under the terms of the GNU General Public License as published by  
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SCB_H
#define SCB_H

enum fhandler {
	HANDLER_MEMMANAGE 	= 4,
	HANDLER_BUSFAULT 	= 5,
	HANDLER_USAGEFAULT 	= 6,
	HANDLER_SVCCALL 	= 11,	// hier nicht drann rumspielen wenn ein RTOS genutzt wird
	HANDLER_PENDSV 		= 14,	// hier nicht drann rumspielen wenn ein RTOS genutzt wird
	HANDLER_SYSTICK 	= 15,	// hier nicht drann rumspielen wenn ein RTOS genutzt wird
};

enum scb_vtor_place {
	SCB_VTOR_SRAM = 1 << 29,	// Vektortable im SRAM
	SCB_VTOR_CODE = 0,			// Vektortable im Flash
};

/** \brief div0 trap aktivieren, lst dann einen UsageFault aus
*/
void scb_en_div0_trap(void);

/** \brief unaligned Speicherzugriffe trap aktivieren, lst dann einen UsageFault aus
	Eine Aktivierung ist nicht empfohlen, denn der GCC nutzt unaligned Zugriffe!
*/
void scb_en_unaligned_trap(void);

/**	\brief Vektor Table offset setzen
	\param addr Startadresse der Vektor Table, bit 7 .. 0 mssen 0 sein (256 byte alignment)
	\param place Speicherberiehc in der sich die tabelle befindet \see enum scb_vtor_place
*/
void scb_vtor(unsigned int addr, enum scb_vtor_place place);

/** \brief System Handler einschalten und prio setzen
	\param handle zu aktivierneder handler \see enum fhandler
	\param preempt_prio preemption prio
	\param sub_prio	sub prio
*/
void scb_enable_faulthandler(enum fhandler handle, unsigned int preempt_prio, unsigned int sub_prio);

/** \brief wenn in anderen handlern ein hardFault aufgetreten ist kommt ein forced HardFault
	\return 1 wenn HardFault was forced, sonst 0
*/
unsigned int scb_hfault_was_forced(void);

/** \brief Faultbits im CFSR (+ Unterregister) werden als String ausgegeben
*/
void scb_print_cfsr_faults(void);

/** \brief Einzelausgaben der Faultregister \see scb_print_cfsr_faults
*/
void scb_print_ufsr_faults(void);
void scb_print_bfsr_faults(void);
void scb_print_mmsr_faults(void);

/** \brief wenn valid, dann Adresse anzeigen
*/
void scb_print_busfaultaddr(void);

/** \brief wenn valid, dann Adresse anzeigen
*/
void scb_print_mmfaultaddr(void);

#endif//SCB_H