/* 
 * This file is part of the ARM Cortex-M Fault distribution.
 * Copyright (c) 2018 Martin Wende.
 * 
 * This program is free software: you can redistribute it and/or modify  
 * it under the terms of the GNU General Public License as published by  
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/** \brief 
  Ausgeben von chars ber den SWO Pin per ITM (Tracecell).
*/

#ifndef SWO_H
#define SWO_H

/** \brief SWO print aktivieren
    \param swo_freq gewnschte Frequenz der Ausgabe am SWO Pin in Hz
    \param cpu_freq CPU Takt in Hz
*/
void swo_init(unsigned int swo_freq, unsigned int cpu_freq);

/** \brief Stimulus Port ein oder aus schalten
    \param stimulus_port Stimulus Port der aktiviert werden soll (0 ... 31)
    \param onoff Stimulus Port aktivieren bei !=0, sonst deaktivieren
*/
void swo_manage_stimulus_port(int stimulus_port, int onoff);

/** \brief Zeichen ber Stimulus Port ausgeben
    \param stimulus_port Stimulus Port ber den ausgegeben soll (0 ... 31)
    \param c auszugebendes Zeichen
*/
void swo_printchar(int stimulus_port, char c);

#endif//SWO_H