#include <stdarg.h>
#include "../periph/usart.h"

static unsigned int ubase = 0;
static void (*byteout_p)(unsigned int, char);

void deprintf_setusart(unsigned int usart_base, void (*byteout)(unsigned int, char)){
	ubase = usart_base;
	byteout_p = byteout;
}

void deprintf_setbaud(unsigned int busclk, unsigned int baudrate){

	usart_rebaud(ubase, baudrate, busclk);
}

static void print_hex(unsigned int val, int do_prefix, int width){
	
	char *hex_mask = "0123456789abcdef";
	int i;

	if (do_prefix) {
		byteout_p(ubase, '0');
		byteout_p(ubase, 'x');
	}

	// Nibble von links nach rechts durchgehen: 0x76543210
	for (i = 7; i >= 0; i--) {
		unsigned int nibble = (val >> (4 * i)) & 0xf;

		// ggf. fhrende Nullen berspringen
		if (!nibble && i >= width){
			continue;
		}

		byteout_p(ubase, hex_mask[nibble]);

		// nach erster Ausgabe keine weiteren Nullen ignorieren
		width = 8;
	}
}

static void print_uint(unsigned int val){

	char out_uint[11];
	int i = 0;
	
	do{
		out_uint[i] = (val%10) + '0';
		val = val/10;
		i++;
	}while (val);
	
	for (i--; i >= 0; i--){
		byteout_p(ubase, out_uint[i]);
	}
}

static void stringout (unsigned int ubase, char *str){
	
	while(*str){
		byteout_p(ubase, *str++);
	}
};

__attribute__ ((format(printf,1,2)))
void deprintf(char *format, ...){
	
	if (0 == ubase){
		return;
	}
	
	va_list ap;
	va_start(ap, format);

	while (*format) {
		if (*format != '%') {
			byteout_p(ubase, *format++);
			continue;
		}

		format++;

		if (!*format) {
			byteout_p(ubase, '%');
			break;
		}

		switch (*format) {
			case 'c':
				byteout_p(ubase, va_arg(ap, unsigned int));
				break;
			case 's':
				stringout(ubase, va_arg(ap, char*));
				break;
			case 'p':
				print_hex(va_arg(ap, unsigned int), 1, 8);
				break;
			case 'x':
				print_hex(va_arg(ap, unsigned int), 1, 1);
				break;
			case 'X':
				print_hex(va_arg(ap, unsigned int), 1, 8);
				break;
			case 'i':
				;
				unsigned int val = va_arg(ap, unsigned int);
				if (val & 0x80000000){
					val = ~val;
					val += 1;
					byteout_p(ubase, '-');
				}
				print_uint(val);
				break;
			case 'u':
				print_uint(va_arg(ap, unsigned int));
				break;
			case '%':
				byteout_p(ubase, '%');
				break;
			default:
				byteout_p(ubase, '%');
				byteout_p(ubase, *format);
		}

		format++;
	}

	va_end(ap);
}