#ifndef _FIFO_H_
#define _FIFO_H_

/** \brief ein FIFO Objekt
*/
struct fifo_vars {
	volatile char *fbuf;			/**< Buffer des FIFO Inhalts */
	unsigned int indexmax;			/**< maximaler Index des Array des Buffers */
	volatile unsigned int head;		/**< head Index des Buffers */
	volatile unsigned int tail;		/**< tail Index des BUffers */
};

/** \brief Initialisierung eines FIFO Objekts

	\param fifo Zeiger auf ein FIFO Objekt
	\param buffer Zeiger auf einen Speicherberich der Gre len
	\param len Gre der FIFO
*/
void fifo_init(struct fifo_vars *fifo, char *buffer, unsigned int len);

/** \brief Lscht den Inhalt der FIFO

	\param fifo Zeiger auf ein FIFO Objekt
*/
void fifo_clear(struct fifo_vars *fifo);

/** \brief speichert ein Zeichen in der FIFO

	\param fifo Zeiger auf ein FIFO Objekt
	\param c zu speicherndes Zeichen
*/
void fifo_put(struct fifo_vars *fifo, char c);

/** \brief speichert ein Zeichen in der FIFO und blockiert wenn die FIFO voll ist

	Darf NICHT vom IRQ Handler aufgerufen werden

	\param fifo Zeiger auf ein FIFO Objekt
	\param c zu speicherndes Zeichen
*/
void fifo_put_block(struct fifo_vars *fifo, char c);

/** \brief liet Zeichen aus der FIFO aus

	\param fifo Zeiger auf ein FIFO Objekt
	\return gelesenes Zeichen oder 0 wenn FIFO leer
*/
char fifo_get(struct fifo_vars *fifo);

/** \brief liet Zeichen aus der FIFO aus und blockiert wenn die FIFO leer ist

	\param fifo Zeiger auf ein FIFO Objekt
	\return gelesenes Zeichen oder 0 wenn FIFO leer
*/
char fifo_get_block(struct fifo_vars *fifo);


//ret 1 = empty
unsigned int fifo_empty(struct fifo_vars *fifo);

#endif//_FIFO_H_
