#include "main.h"

#include <stdint.h>
#include <stdlib.h>
#include <string.h>	
#include <stdio.h>

#include "periph/rcc.h"
#include "periph/usart.h"
#include "periph/gpio.h"

#include "armcore/mpu.h"
#include "armcore/scb.h"
#include "armcore/swo.h"

#include "lib/deprintf.h"

static char usart3_tx_buf[256];
static char usart3_rx_buf[16];
static struct usart_irq usart3_irq;
static uint8_t receive_index = 0;
static char recbuf[2] = {0, 0};

extern void divme(int, int); // liegt in startup.s rum

int main (void) {
	
	// Traps aktivieren und Faulthandler aktivieren
	// Faults haben Prio 1
	scb_en_div0_trap();
	//scb_en_unaligned_trap(); bleibt auskommentiet, denn unaligned ist erlaubt!
	scb_enable_faulthandler(HANDLER_MEMMANAGE, 1, 0);
	scb_enable_faulthandler(HANDLER_BUSFAULT, 1, 0);
	scb_enable_faulthandler(HANDLER_USAGEFAULT, 1, 0);
	
	// MPU aktivieren und Nullpointerschutz
	
	// Region 0 -> 1MB Flash erlauben	
	mpu_set_region(	
		0, 
		0x08000000, // Startadresse
		MPU_REGS_1MBYTE,
		0,			// keine Subregions
		MPU_FLASH, 
		MPU_KRO_URO,
		1);			// Code ausfhrbar
	
	// Region 1 -> SRAM1/2 erlauben	
	mpu_set_region(	
		1, 
		0x20000000, // Startadresse 			
		MPU_REGS_128KBYTE,
		0,			// keine Subregions
		MPU_RAMI, 
		MPU_KRW_URW,
		0);
		
	// Region 2 -> Periph erlauben	
	mpu_set_region(	
		2, 
		0x40000000, // Startadresse 			
		MPU_REGS_512MBYTE,
		0,			// keine Subregions
		MPU_PERI, 
		MPU_KRW_URW,
		0);
		
	// Region 3 -> Cortex Register erlauben
	mpu_set_region(	
		3, 
		0xE0000000, // Startadresse 			
		MPU_REGS_512MBYTE,
		0,			// keine Subregions
		MPU_PERI, 
		MPU_KRW_URW,
		0);
		
	// IVT direkt aus dem Flash lesen anstatt dem Remap
	// Somit kann das 0. MegaByte gesperrt werden fr eine Nullpointererkennung
	scb_vtor(0x08000000, SCB_VTOR_CODE);
	mpu_onoff(1);

	// Debug UART init (USART3) fr deprintf
	rcc_enable_clock(RCC_GPIOB);
	gpio_initpin(PORTB, 10, AF_PP, GPIO_HS, AF7_USART1_TO_3); //TX = PB10
	gpio_initpin(PORTB, 11, AF_PP, GPIO_HS, AF7_USART1_TO_3); //RX = PB11
	rcc_enable_clock(RCC_USART3);
	fifo_init(&(usart3_irq.tx_fifo), usart3_tx_buf, sizeof(usart3_tx_buf));
	fifo_init(&(usart3_irq.rx_fifo), usart3_rx_buf, sizeof(usart3_rx_buf));
	usart_init_irq(&usart3_irq, USART3_INDEX, UART_BAUDRATE, SYS_CLK); 
	deprintf_setusart(USART3_INDEX, usart_putc);
	
	deprintf("----------------------------------------------------------\nBOOTIN!\n");
	
	deprintf("SWO Init");
	swo_init(SWO_CLOCK, SYS_CLK); 				// SWO initialisieren
	swo_manage_stimulus_port(SWO_STIMPORT, 1); 	// SWO StimPort aktivieren
	
	deprintf(" OK\nSWO Testausgabe");
	swo_printchar(SWO_STIMPORT, 'H');
	swo_printchar(SWO_STIMPORT, 'E');
	swo_printchar(SWO_STIMPORT, 'L');
	swo_printchar(SWO_STIMPORT, 'L');
	swo_printchar(SWO_STIMPORT, 'O');
	swo_printchar(SWO_STIMPORT, '\n');
	deprintf(" OK\n");
	
	printf("printf ueber SWO! mit floats! %f\n", 1337.42f);
	
	deprintf("\nWie tief moechten Sie heute ins Klo greifen?\n");
	deprintf("ud -> div0 (usagefault)\n");
	deprintf("uu -> undefined Instruction (usagefault)\n");
	deprintf("mn -> Nullpointerzugriff (Daten) (MPU Fault)\n");
	deprintf("mi -> Nullpointerzugriff (Instruction) (MPU Fault)\n");
	deprintf("input: ");
 
	while(1) {
		
		recbuf[receive_index] = usart_getc(USART3_INDEX);
		deprintf("%c", recbuf[receive_index]);
		
		if (recbuf[0] == 'u'){
			// UsageFaults
			receive_index = 1;
			switch (recbuf[1]){
				
				case 'd':
					divme(5, 0); // 5/0
					break;
					
				case 'u':
					asm("udf #1"); // udf lassen sich nutzen um illegale Programmzustnde abzufangen
					break;
			}
		}
		
		if (recbuf[0] == 'm'){
			// MPUFaults
			receive_index = 1;
			volatile uint32_t klogriff_data;
			void (*klogriff_instr)(void);
			switch (recbuf[1]){
				
				case 'n':
					klogriff_data = *((uint32_t*)0x00000000);
					(void)klogriff_data;
					break;
					
				case 'i':
					klogriff_instr = (void (*)(void))0x00000000;
					klogriff_instr();
					break;
			}
		}
		
	}
	
	return 0;
}


