#include "gpio.h"

struct gpio {
	unsigned int MODER;
	unsigned int OTYPER;
	unsigned int OSPEEDER;
	unsigned int PUPDR;
	unsigned int IDR;
	unsigned int ODR;
	unsigned int BSRR;
	unsigned int LCKR;
	unsigned int AFRL;
	unsigned int AFRH;
};

// 4 Bits per Alternate Function
#define AFR_BITS 4
#define AFR_MSK  0xF

// ansonsten 2 Bits pro Funktion
#define DUALBIT_MSK 0b11

// zweigeteiltes Register fr SET/RESET
#define BSSR_SET_POS 0
#define BSSR_RES_POS 16

// ein enum initpin definiert mit Bitgefrickel den Pinzustand fr ein einfacheres benutzen der Lib
// sorgt dafr aber fr etwas mehr Komplexitt in der Lib
// codierung des passenden MODE Registers
#define ENUM_REGMODER_MSK 0b11000
#define ENUM_REGMODER_POS 3
// codierung des passenden MODE Bits
#define ENUM_BITMODER_MSK 0b11
#define ENUM_BITMODER_POS 0
// codierung der Ausgangstreiber
#define ENUM_OTYPE_MSK 0b100
#define ENUM_OTYPE_POS 2
// codierung der Pullwiderstnde
#define ENUM_PUPDR_MSK 0b11
#define ENUM_PUPDR_POS 0

#define AFR_PINS_PER_REG 8

int gpio_initpin(unsigned int gpio_base, uint8_t pin_nbr, enum initpin mode, enum pinspeed speed, enum altfuncs altfunc){
	
	volatile struct gpio * const gpio = (struct gpio *)gpio_base;
	
	// 16 Pins pro Port
	if (pin_nbr >= 16){
		return -1;
	}
	
	// Mode (Input, GP Out, Alternate function, Analog)
	uint32_t reg_mode = (mode & ENUM_REGMODER_MSK) >> ENUM_REGMODER_POS;
	gpio->MODER &= ~(ENUM_BITMODER_MSK << (pin_nbr*2));
	gpio->MODER |= (reg_mode << (pin_nbr*2));
	
	// Ausgangstreiber (Pushpull, Opendrain)
	uint32_t reg_ppod = (mode & ENUM_OTYPE_MSK) >> ENUM_OTYPE_POS;
	gpio->OTYPER &= ~(1 << pin_nbr);
	gpio->OTYPER |= (reg_ppod << pin_nbr);
	
	// Slewrate (Lowspeed, Mediumspeed, Fastspeed, Highspeed)
	gpio->OSPEEDER &= ~(DUALBIT_MSK << (pin_nbr*2));
	gpio->OSPEEDER |= (speed << (pin_nbr*2));
	
	// Pullup, Pulldown
	uint32_t reg_pull = mode & ENUM_PUPDR_MSK;
	gpio->PUPDR &= ~(ENUM_PUPDR_MSK << (pin_nbr*2));
	gpio->PUPDR |= (reg_pull << (pin_nbr*2));
	
	// Altfunc schreiben
	volatile unsigned int *afr = &(gpio->AFRL);
	
	// passendes AFRx Register und Bitgruppenposition raussuchen
	if (pin_nbr >= AFR_PINS_PER_REG){
		afr = &(gpio->AFRH);
		pin_nbr -= AFR_PINS_PER_REG;
	}
	
	*afr &= ~(AFR_MSK << (pin_nbr*AFR_BITS));
	*afr |=  (altfunc << (pin_nbr*AFR_BITS));
	
	return 0;
}

void gpio_onoff(unsigned int gpio_base, uint8_t pin_nbr, uint8_t onoff){

	volatile struct gpio * const gpio = (struct gpio *)gpio_base;
	
	if (onoff){
		gpio->BSRR = 1 << (pin_nbr + BSSR_SET_POS);
	} else {
		gpio->BSRR = 1 << (pin_nbr + BSSR_RES_POS);
	}
}

