#ifndef GPIO_H
#define GPIO_H

#include <stdint.h>

#define PORTA 0x40020000
#define PORTB 0x40020400
#define PORTC 0x40020800
#define PORTD 0x40020C00
#define PORTE 0x40021000
#define PORTF 0x40021400
#define PORTG 0x40021800
#define PORTH 0x40021C00
#define PORTI 0x40022000

/** \brief enum der mglichen Pinzustnde 
	GP = general-purpose, AF = alternate function, 
	PP = push-pull, OD = open-drain, 
	LS = low speed, MS = medium speed, FS = fast speed, HS = high speed, 
	PU = pull-up, PD = pull-down.
*/
enum initpin {
	GPO_PP = 		0b01000,
	GPO_PP_PU = 	0b01001,
	GPO_PP_PD = 	0b01010,
	GPO_OD = 		0b01100,
	GPO_OD_PU = 	0b01101,
	GPO_OD_PD = 	0b01110,
	AF_PP = 		0b10000,
	AF_PP_PU = 		0b10001,
	AF_PP_PD = 		0b10010,
	AF_OD = 		0b10100,	
	AF_OD_PU = 		0b10101,
	AF_OD_PD = 		0b10110,
	GPI_FLOAT = 	0b00000,
	GPI_PU = 		0b00001,
	GPI_PD = 		0b00010,
	GP_ANALOG = 	0b11000,
};

/** \brief enum der mglichen Pingeschwindigkeiten  
*/
enum pinspeed {
	GPIO_LS = 0b00,
	GPIO_MS = 0b01,
	GPIO_FS = 0b10,
	GPIO_HS = 0b11,
};

enum altfuncs {
	AF0_SYS 		= 0,
	AF1_TIM1_TO_2 	= 1,
	AF2_TIM3_TO_5 	= 2,
	AF3_TIM8_TO_11	= 3,
	AF4_I2C1_TO_3 	= 4,
	AF5_SPI1_TO_2 	= 5,
	AF6_SPI3 		= 6,
	AF7_USART1_TO_3 = 7,
	AF8_USART4_TO_6 = 8,
	AF9_CAN1_TO_2_TIM12_TO_14 = 9,
	AF10_OTGFS_OTGHS = 10,
	AF11_ETH 		= 11,
	AF12_FSMC_SDIO_OTGHS = 12,
	AF13_DCMI 		= 13,
	AF14 			= 14,
	AF15_EVENTOUT 	= 15,
};

/** \brief Initialisiert einen GPIO Pin
	\param gpio_base Basisadresse eines GPIO Blocks des zu initialisierenden Pins
	\param pin_nbr Pin Nummer des zu initialisierenden Pins
	\param mode gewnschter Initialisierungszustand des Pins
	\param speed spezifiziert die Slewrate wenn mode auf Ausgang gestellt wurde
	\param altfunc spezifiziert die alternate function wenn initpin auf AF_x gestellt wurde
*/
int gpio_initpin(unsigned int gpio_base, uint8_t pin_nbr, enum initpin mode, enum pinspeed speed, uint8_t altfunc);

void gpio_onoff(unsigned int gpio_base, uint8_t pin_nbr, uint8_t onoff);

#endif//GPIO_H