#ifndef USART_H
#define USART_H

#include <stdint.h>

#define USART_USE_IRQ	1
#define USART_USE_DMA	0

#define USART1_BASE_INT 0x40011000
#define USART2_BASE_INT 0x40004400
#define USART3_BASE_INT 0x40004800
#define UART4_BASE_INT  0x40004C00
#define UART5_BASE_INT  0x40005000
#define USART6_BASE_INT 0x40011400

//wird auch pse in der usart_asm.S benutzt!
#define USART1_INDEX	1
#define USART2_INDEX	2
#define USART3_INDEX	3
#define UART4_INDEX		4
#define UART5_INDEX		5
#define USART6_INDEX	6

#if USART_USE_DMA
	#include "periph/dma.h"
#endif


#if USART_USE_IRQ

#include "lib/fifo.h"
struct usart_irq {
	unsigned int usart_base_addr;
	struct fifo_vars tx_fifo;
	struct fifo_vars rx_fifo;
};

//"normales" Init mit IRQ, braucht 2 initialisierte FIFOs
void usart_init_irq(struct usart_irq *idx, unsigned int usart_base, unsigned int baudrate, unsigned int bus_clk);
#endif

//ret 1 bei Fehler
#if USART_USE_DMA
int usart_dmasend(
	unsigned int usart_base,				//Welcher USART
	enum dma_periph peri,					//Welcher DMA/Stream/Channel
	uint8_t *buf,							//Buf zum senden
	uint16_t buflen,						//Elemente im Buf
	void (*callb)(unsigned int, void*),	//Callback be auftretendem DMA IRQ
	void *callb_data						//Daten fr den DMA Callback
	);
#endif

void usart_rebaud(unsigned int usart_base, unsigned int baudrate, unsigned int bus_clk);

//Init mit Polling fr Faulthandler, deaktiviert IRQs und DMA
void usart_init_poll(unsigned int usart_base, unsigned int baudrate, unsigned int bus_clk);

#if USART_USE_IRQ
void usart_putc(unsigned int usart_base, char c);
void usart_puts(unsigned int usart_base, char *str);
char usart_getc(unsigned int usart_base);
#endif

void usart_poll_putc(unsigned int usart_base, char c);

#endif//USART_H