/* 
 * This file is part of the ARM Cortex-M Fault distribution.
 * Copyright (c) 2018 Martin Wende.
 * 
 * This program is free software: you can redistribute it and/or modify  
 * it under the terms of the GNU General Public License as published by  
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include "disasm.h"

#include "lib/deprintf.h"

#include <stdint.h>

static char *regnames[] = {
	"R0", "R1", "R2",  "R3",  "R4",  "R5", "R6", "R7", 
	"R8", "R9", "R10", "R11", "R12", "SP", "LR", "PC",
};

void disasm_print(unsigned int faultaddr){

	// Pointer aus Adrsse basteln
	uint16_t *instr = (uint16_t*)faultaddr;
	uint16_t *instr2 = (uint16_t*)(faultaddr+2);
	
	// udf angucken (0xde__)
	if ((*instr & 0xFF00) == 0xde00){
		deprintf("udf #%u (%x)\n", *instr & 0x00FF, *instr & 0x00FF);
	}
	
	// udf.w angucken (0xf7f0 a___)
	if ((*instr == 0xf7f0) && ((*instr2 & 0xF000) == 0xa000)){
		deprintf("udf.w #%u (%x)\n", *instr2 & 0x0FFF, *instr2 & 0x0FFF);
	}
	
	// sdiv (0xFB9_ 0xF_F_)
	if (((*instr & 0xFFF0) == 0xFB90) && ((*instr2 & 0xF0F0) == 0xF0F0)){
		uint8_t rn = *instr & 0x000F;
		uint8_t rd = (*instr2 & 0x0F00) >> 8;
		uint8_t rm = *instr2 & 0x000F;
		deprintf("sdiv %s, %s, %s\n", regnames[rd], regnames[rn], regnames[rm]);
	}
	
	// udiv (0xFBB_ 0xF_F_)
	if (((*instr & 0xFFF0) == 0xFBB0) && ((*instr2 & 0xF0F0) == 0xF0F0)){
		uint8_t rn = *instr & 0x000F;
		uint8_t rd = (*instr2 & 0x0F00) >> 8;
		uint8_t rm = *instr2 & 0x000F;
		deprintf("udiv %s, %s, %s\n", regnames[rd], regnames[rn], regnames[rm]);
	}
}