/* 
 * This file is part of the ARM Cortex-M Fault distribution.
 * Copyright (c) 2018 Martin Wende.
 * 
 * This program is free software: you can redistribute it and/or modify  
 * it under the terms of the GNU General Public License as published by  
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

ENTRY(_startup)
SECTIONS{

	/* Flash Anfang (1MB Gre)*/
	. = 0x08000000;
	_flash_start = .;
	
	/* Das Programm kommt natrlich in den Flash */
	.text : {
		*(.text.vectors) /* NVIC Vektoren MSSEN an den Start! */
		*(.text*)
	}
	
	/* Readonly Zeuch (= Konstanten) kommt auch in den Flash */
	. = ALIGN(4);
	.rodata : {
		*(.rodata*)
	}
	
	/*Hier liegt .data im Flash zum Kopieren */
	. = ALIGN(4);
	_rom_data_start = .;
	_rom_data2_start = . + _data_size;
   
	/* Zum rausfinden ob Programm zu gro fr den Flash */
	_flash_end = . + _data2_size;
	
	
	
	/* RAM ab jetzt (112k Gre)*/
	. = 0x20000000;
	
	/* .data wird hierher gelinkt, aber im Flash gespeichert */
	.data : AT( _rom_data_start ) {
		_data_start = .;
		*(.data*)
		_data_end = .;
	}
	_data_size = _data_end - _data_start;
	
	. = ALIGN(4);
	.bss : {
		_bss_start = .;
		*(.bss*)
		*(common*)
		*(.common*)
		_bss_end = .;
	}
	_bss_size = _bss_end - _bss_start;
	
	/* Nach dem Programm und seinen statischen Daten
	kommt der Heap mit den dynamischen Daten */
	. = ALIGN(4);
	end = .;
	
	
	/* RAM2 ab jetzt fr DMA Zeuchs (16k Gre)*/
	. = 0x2001C000;
	
	/* .data wird hierher gelinkt, aber im Flash gespeichert */
	.data2 : AT( _rom_data2_start ) {
		_data2_start = .;
		*(.dma_data*)
		_data2_end = .;
	}
	_data2_size = _data2_end - _data2_start;
	
	. = ALIGN(4);
	.bss2 : {
		_bss2_start = .;
		*(.dma_bss*)
		_bss2_end = .;
	}
	_bss2_size = _bss2_end - _bss2_start;
	
	
	/* Checks */
	ASSERT(_flash_end - _flash_start < 0x100000, "Flash zu klein!")
	ASSERT(_data_size + _bss_size < 0x1C000, "RAM zu klein!")
	ASSERT(_data2_size + _bss2_size < 0x4000, "RAM2 zu klein!")
}